@@ -1,10 +1,12 @@
+use crate::black_list::contains;
 use crate::configuration::Configuration;
 use crate::page::Page;
+use crate::utils::{fetch_page_html, Client};
+
+use rayon::ThreadPool;
 use rayon::ThreadPoolBuilder;
 use robotparser_fork::RobotFileParser;
 
-use crate::black_list::contains;
-use crate::utils::{fetch_page_html, Client};
 use std::collections::HashSet;
 use std::{sync, thread, time::Duration};
 
@@ -35,8 +37,6 @@ pub struct Website<'a> {
     pub on_link_find_callback: fn(String) -> String,
     /// Robot.txt parser holder
     robot_file_parser: RobotFileParser<'a>,
-    // configured the robots parser
-    configured_robots_parser: bool,
     // fetch client
     client: Client,
 }
@@ -49,7 +49,6 @@ impl<'a> Website<'a> {
 
         Self {
             configuration: Configuration::new(),
-            configured_robots_parser: false,
             domain: domain.to_string(),
             links,
             links_visited: HashSet::new(),
@@ -65,58 +64,70 @@ impl<'a> Website<'a> {
         &self.pages
     }
 
+    /// crawl delay getter
+    pub fn get_delay(&self) -> Duration {
+        Duration::from_millis(self.configuration.delay)
+    }
+
     /// configure the robots parser on initial crawl attempt and run
     pub fn configure_robots_parser(&mut self) {
-        if self.configuration.respect_robots_txt && !self.configured_robots_parser {
-            self.configured_robots_parser = true;
+        if self.configuration.respect_robots_txt && self.robot_file_parser.mtime() == 0 {
             self.robot_file_parser.user_agent = self.configuration.user_agent.to_string();
             self.robot_file_parser.read();
-
-            // returns the crawl delay in seconds
             self.configuration.delay = self
                 .robot_file_parser
-                .get_crawl_delay(&self.robot_file_parser.user_agent)
-                .unwrap_or_else(|| Duration::from_millis(self.configuration.delay))
+                .get_crawl_delay(&self.robot_file_parser.user_agent) // returns the crawl delay in seconds
+                .unwrap_or(self.get_delay())
                 .as_millis() as u64;
         }
     }
 
+    /// configure http client
+    pub fn configure_http_client(&mut self, user_agent: Option<String>) {
+        self.client = Client::builder()
+            .user_agent(user_agent.unwrap_or(self.configuration.user_agent.to_string()))
+            .pool_max_idle_per_host(0)
+            .build()
+            .expect("Failed building client.")
+    }
+
+    /// configure rayon thread pool
+    fn create_thread_pool(&mut self) -> ThreadPool {
+        ThreadPoolBuilder::new()
+            .num_threads(self.configuration.concurrency)
+            .build()
+            .expect("Failed building thread pool.")
+    }
+
     /// Start to crawl website
     pub fn crawl(&mut self) {
         self.configure_robots_parser();
-        let delay = Duration::from_millis(self.configuration.delay);
-        let pool = ThreadPoolBuilder::new()
-            .num_threads(self.configuration.concurrency)
-            .build()
-            .expect("Failed building thread pool.");
-        self.client = Client::builder()
-            .user_agent(self.configuration.user_agent)
-            .pool_max_idle_per_host(0)
-            .build()
-            .expect("Failed building client.");
+        self.configure_http_client(None);
+        let delay = self.get_delay();
+        let pool = self.create_thread_pool();
+        let on_link_find_callback = self.on_link_find_callback;
 
         // crawl while links exists
         while !self.links.is_empty() {
             let mut new_links: HashSet<String> = HashSet::new();
             let (tx, rx) = sync::mpsc::channel();
-            let on_link_find_callback = self.on_link_find_callback;
 
-            self.links
-                .iter()
-                .filter(|link| self.is_allowed(link))
-                .for_each(|link| {
-                    self.log(&format!("- fetch {}", link));
-                    let thread_link = link.to_string();
-                    let tx = tx.clone();
-                    let cx = self.client.clone();
+            for link in self.links.iter() {
+                if !self.is_allowed(link) {
+                    continue;
+                }
+                self.log(&format!("- fetch {}", &link));
+                let thread_link = link.to_string();
+                let tx = tx.clone();
+                let cx = self.client.clone();
 
-                    pool.spawn(move || {
-                        let link_result = on_link_find_callback(thread_link);
-                        let html = fetch_page_html(&link_result, &cx).unwrap_or("".to_string());
-                        tx.send(Page::new(&link_result, &html)).unwrap();
-                        thread::sleep(delay);
-                    });
+                pool.spawn(move || {
+                    let link_result = on_link_find_callback(thread_link);
+                    let html = fetch_page_html(&link_result, &cx).unwrap_or_default();
+                    tx.send(Page::new(&link_result, &html)).unwrap();
+                    thread::sleep(delay);
                 });
+            }
 
             drop(tx);
 
@@ -124,7 +135,7 @@ impl<'a> Website<'a> {
                 let url = page.get_url();
                 self.log(&format!("- parse {}", url));
                 new_links.extend(page.links(&self.domain));
-                self.links_visited.insert(String::from(url));
+                self.links_visited.insert(url.to_string());
                 self.pages.push(page);
             });
 
